/**
 * easySchema - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.ow2.easywsdl.schema.util;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.ow2.easywsdl.schema.api.XmlException;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;


/**
 * @author Nicolas Salatge - eBM WebSourcing
 */
public class SourceHelper {

    private final static ThreadLocal<DocumentBuilder> documentBuilderThreadLocal = new ThreadLocal<DocumentBuilder>() {

        @Override
        protected DocumentBuilder initialValue() {
            try {
                final DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory
                        .newInstance();
                documentBuilderFactory.setNamespaceAware(true);
                final DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
                return documentBuilder;
            } catch (final ParserConfigurationException e) {
                throw new RuntimeException("Failed to create DocumentBuilder", e);
            }
        }
    };

    private final static ThreadLocal<Transformer> transformerWithoutXmlDeclarationThreadLocal = new ThreadLocal<Transformer>() {

        @Override
        protected Transformer initialValue() {
            try {
                final Transformer transformer = TransformerFactory.newInstance().newTransformer();

                final Properties props = new Properties();
                props.put(OutputKeys.OMIT_XML_DECLARATION, "yes");
                transformer.setOutputProperties(props);
                return transformer;

            } catch (final TransformerConfigurationException e) {
                throw new RuntimeException("Failed to create Transformer", e);
            }
        }
    };

    /**
     * Creates a new instance of {@link SourceHelper}
     * 
     */
    private SourceHelper() {
        super();
    }

    /**
     * Create a Source from a DOM document.
     * 
     * @param document
     *            A DOM document
     * @return the Source built from the DOM document
     */
    public static DOMSource createSource(final Document document) {
        return new DOMSource(document);
    }

    /**
     * Create a DOM Document from a Source.
     * 
     * @param s
     *            the ${link {@link Source}
     * @return the DOM document
     * @throws PEtALSCDKException
     */
    public static Document createDocument(final Source s) throws XmlException {

        try {
            final Document document = SourceHelper.documentBuilderThreadLocal.get().newDocument();
            final DOMResult domResult = new DOMResult(document);
            SourceHelper.transformerWithoutXmlDeclarationThreadLocal.get().transform(s, domResult);
            return document;
        } catch (final TransformerException e) {
            throw new XmlException(e);
        }
    }

    public static DOMSource convertInputSource2DOMSource(final InputSource inputSource)
            throws XmlException {
        DOMSource res = null;
        try {
            final DocumentBuilder db = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            final Document document = db.parse(inputSource);
            res = new DOMSource(document);
        } catch (final ParserConfigurationException e) {
            throw new XmlException(e);
        } catch (final SAXException e) {
            throw new XmlException(e);
        } catch (final IOException e) {
            throw new XmlException(e);
        }
        return res;
    }

    public static InputSource convertDOMSource2InputSource(final DOMSource inputSource)
            throws XmlException {
        InputSource res = null;
        try {
            final ByteArrayOutputStream os = new ByteArrayOutputStream();
            final StreamResult streamResult = new StreamResult(os);
            final Transformer transformer = TransformerFactory.newInstance().newTransformer();
            transformer.transform(inputSource, streamResult);
            final InputStream is = new java.io.ByteArrayInputStream(os.toByteArray());
            res = new InputSource(is);
        } catch (final TransformerException e) {
            throw new XmlException(e);
        }
        return res;
    }

    public static DOMSource cloneSource(final DOMSource input) throws XmlException {
        final Document doc = SourceHelper.createDocument(input);
        return SourceHelper.createSource(doc);
    }

}
